#!/usr/bin/perl
#
# Irssi Quotes System version 1.2 (05 Jul 2006)
#
# Well, I was searching http://scripts.irssi.org and I found 0 results matching *quote*.
# So this is where I got the idea from to make this quotes system, in case you are interested.
# Why using another IRC client, or bot, or whatever, to have quotes system when you can use Irssi ? *G*
# What do you need to know to use it ? You need to know how to import the sql query, and how to load the script!
# Assuming you have MySQL, Perl, DBI, and of course Irssi installed on your machine ;-)
#  
# So, here's the SQL query:
#
# CREATE TABLE irssi_quotes (
#   qid INTEGER UNSIGNED PRIMARY KEY NOT NULL AUTO_INCREMENT,
#   nick TEXT NOT NULL,
#   quote TEXT NOT NULL,
#   deleted TINYINT(1) UNSIGNED NOT NULL DEFAULT 0
# );
#
# And this is how it should look like:
#
# mysql> show columns from irssi_quotes;
# +---------+---------------------+------+-----+---------+----------------+
# | Field   | Type                | Null | Key | Default | Extra          |
# +---------+---------------------+------+-----+---------+----------------+
# | qid     | int(10) unsigned    | NO   | PRI | NULL    | auto_increment |
# | nick    | text                | NO   |     |         |                |
# | quote   | text                | NO   |     |         |                |
# | deleted | tinyint(1) unsigned | NO   |     | 0       |                |
# +---------+---------------------+------+-----+---------+----------------+
# 4 rows in set (0.00 sec)
#
# You *MUST* edit these variables in case you want the quotes system up and running:
#
#         $database - the database name which will be used for the quotes system
#         $username - the database username
#         $password - the password
#
# After you do that, put the script in your irssi scripts directory and do /script load irssi_quotes.pl
#
# *** NOTE: By default only ops can use the !addquote command.
#	    If you want to change that, edit the value of $addqlevel
#
# Implemented commands that can be used in any channel where the Irssi client is:
#
#   !help - To send this info via notice to the one who requested it.
#   !quote [id] - Shows the quote with that id, or a random quote if none is used.
#   !search <string> - Searches for *string* - SQL-style wildcards.
#   !whoset <id> - Shows the author of the quote under that id.
#   !addquote <text> - Adds a quote in the database.
#   !db - Prints number of non-deleted quotes in the database.
#
# Commands for private use only, directly on the Irssi client of course:
#
#   /delquote <id> - Deletes a quote (actually it just marks the quote as deleted,
#				      nobody will be able to see it unless you restore it)
#   /restore <id> - Restores a quote.
#
# Credits:
#
#    Dimitar Tnokovski aka Mitko [mitko _at_ surferz _dot_ info] [ Mitko @ Undernet IRC ]
#    And thanks to Tincan and |nemo| who helped me with some sql stuff.
#
#
# Feel free to edit it, but leave the credits please. If you find a bug, please send me a note! :)
#
#                                                              - Mitko

use Irssi;
use DBI;
use vars qw($VERSION %IRSSI);
use Irssi qw(command_bind active_win);

$VERSION = "1.1";
%IRSSI = (
    author 	=> 'Dimitar Tnokovski aka Mitko',
    contact 	=> 'mitko\@surferz.info ',
    name 	=> 'irssi_quotes',
    description => 'Irssi quotes system that will allow others to add, search and view quotes stored in the mysql database',
    license     => 'Public Domain',
    url 	=> 'http://surferz.info/?page=irssi:quotes',
    changed 	=> 'Wed Jul 5 12:19:27 CEST 2006'
);

# 1 - only +o users can add quotes
# 2 - only +o and +v users can add quotes 
# 3 - everyone can add quotes

$addqlevel = ('1');

$database = ('yourdbname');
$username = ('dbuser');
$password = ('dbpass');

$dbhost = ('localhost');
$socket = ('/var/run/mysql/mysql.sock');
# this is the default one, if it doesn't work see it's value in the mysql config: my.cnf


sub event_privmsg {
my ($server, $data, $nick, $mask, $target) =@_;
my ($target, $text) = $data =~ /^(\S*)\s:(.*)/;

if ($text =~ /^!quote */i && $target =~ /#/i)
{
        my ($showquote) = $text =~ /^!quote (.*)/;

        if (length($showquote) < 1) {

                $dbh = DBI->connect("DBI:mysql:$database:$dbhost;mysql_socket=$socket","$username","$password")
                or die "Could not connect to database: " . DBI->errstr;

                $sth = $dbh->prepare("SELECT * FROM `irssi_quotes` WHERE deleted = 0 order by rand() limit 1;")
                or die "Can not prepare $statement: $dbh->errstr\n";

                $rv = $sth->execute() or die "Can not execute the query: $sth->errstr\n";
                if ($rv >= 1)
                {
                        while (my $ref = $sth->fetchrow_hashref())
                        {
                                $server->command ( "msg $target Quote: $ref->{'qid'} --> $ref->{'quote'}\n" );
                        }
                }
                else
                {
                   $server->command ( "notice $nick Can not show a random quote.\n" );
                }
                $sth->finish;

        }

        else {
                $dbh = DBI->connect("DBI:mysql:$database:$dbhost;mysql_socket=$socket","$username","$password")
                or die "Could not connect to database: " . DBI->errstr;

                $sth = $dbh->prepare("SELECT * FROM `irssi_quotes` WHERE qid = ? and deleted = 0;")
                or die "Can not prepare $statement: $dbh->errstr\n";

                $rv = $sth->execute($showquote) or die "Can not execute the query: $sth->errstr\n";
                if ($rv >= 1)
                {
                        while (my $ref = $sth->fetchrow_hashref())
                        {
                                $server->command ( "msg $target Quote: $ref->{'qid'} --> $ref->{'quote'}\n" );
                        }
                }
                else
                {
                   $server->command ( "notice $nick Quote not found.\n" );
                }
                $sth->finish;
        }
}

elsif ($text =~ /^!search */i && $target =~ /#/i)
{

        my ($searchchan) = $text =~ /^!search (.*)/;
        my $schchan = "%" . $searchchan . "%";
        $dbh = DBI->connect("DBI:mysql:$database:$dbhost;mysql_socket=$socket","$username","$password")
                or die "Couldn't connect to database: " . DBI->errstr;
        $sth = $dbh->prepare("SELECT * FROM `irssi_quotes`
                        WHERE quote like ? and deleted = 0 order by qid desc limit 20;")
                or die "Cant prepare $statement: $dbh->errstr\n";
        $rv = $sth->execute($schchan)
                or die "cant execute the query: $sth->errstr\n";
        if ($rv >= 1)
        {

                my $results = "";
                while (my $ref = $sth->fetchrow_hashref())
                {
                        $results .= $ref->{'qid'} . " ";
                }
                if ($rv >= 20)
                {
                        $results .= " ... ";
                }
                $server->command ( "notice $nick Quotes matching your search: $results\n" );
        }
        else
        {
           $server->command ( "notice $nick Sorry, no quotes match your search.\n" );
        }
        $sth->finish;

}


elsif ($text =~ /^!db */i && $target =~ /#/i)
{
        $dbh = DBI->connect("DBI:mysql:$database:$dbhost;mysql_socket=$socket","$username","$password")
                or die "Could not connect to database: " . DBI->errstr;
        $sth = $dbh->prepare("SELECT count(*) as numrows FROM `irssi_quotes` WHERE deleted = 0;")
                or die "Can not prepare $statement: $dbh->errstr\n";
        $rv = $sth->execute
                or die "Can not execute the query: $sth->errstr\n";
        if ($rv >= 1)
        {
                my $numrows;
                if (my $ref = $sth->fetchrow_hashref())
                {
                        $numrows = $ref->{'numrows'};
                }
                $server->command ( "msg $target There are $numrows active quotes in the database.\n" );
        }
        $sth->finish;

}


elsif ($text =~ /^!whoset */i && $target =~ /#/i)
{
        my ($whoquote) = $text =~ /^!whoset (.*)/;

        $dbh = DBI->connect("DBI:mysql:$database:$dbhost;mysql_socket=$socket","$username","$password")
                or die "Could not connect to database: " . DBI->errstr;

        $sth = $dbh->prepare("SELECT * FROM `irssi_quotes`
                                WHERE qid = ? and deleted = 0;")
                or die "Can not prepare $statement: $dbh->errstr\n";

        $rv = $sth->execute($whoquote) or die "Can not execute the query: $sth->errstr\n";
        if ($rv >= 1)
        {
                while (my $ref = $sth->fetchrow_hashref())
                {
                        $server->command ( "msg $target Author of the quote with id $ref->{'qid'} is $ref->{'nick'}\n" );
                }
        }
        else
        {
           $server->command ( "msg $target No quote found matching your request.\n" );
        }
        $sth->finish;
}


elsif ($text =~ /^!help */i && $target =~ /#/i)
{ 
        $server->command ( "notice $nick Irssi Quotes System v1.1\n" );
        $server->command ( "notice $nick !quote [id] - Shows the quote with that id, or a random quote if none is used.\n" );
        $server->command ( "notice $nick !addquote <text> - Adds a quote in the database.\n" );
        $server->command ( "notice $nick !search <string> - Searches for *string* - SQL-style wildcards.\n" );
        $server->command ( "notice $nick !whoset <id> - Shows the author of the quote under that id.\n" );
        $server->command ( "notice $nick !db - Prints number of non-deleted quotes in the database.\n" );
}


elsif ($text =~ /^!addquote */i && $target =~ /#/i)
{
        if ($addqlevel eq 1 && $server->channel_find($target)->nick_find($nick)->{op}) { $add_it = ('1'); }
        if ($addqlevel eq 2 && $server->channel_find($target)->nick_find($nick)->{voice}
		            || $server->channel_find($target)->nick_find($nick)->{op}) { $add_it = ('1'); }
        if ($addqlevel eq 3) { $add_it = ('1'); }

        if ($add_it eq 1) {

	        my ($quote) = ($text =~ /^!addquote (.*)$/);
	
	        $dbh = DBI->connect("DBI:mysql:$database:$dbhost;mysql_socket=$socket","$username","$password")
	                or die "Could not connect to database: " . DBI->errstr;

	        if (length($quote) > 2)
	        {

	        $sth = $dbh->prepare('INSERT INTO irssi_quotes (nick, quote) VALUES (?, ?)');

	        $rv = $sth->execute($nick, $quote) or die "Can not execute the query: $sth->errstr\n";

	        if ($rv >= 1)
	        {
	                $server->command ( "notice $nick Added quote with id: $sth->{'mysql_insertid'}.\n" );
	                $add_it = ('0');
	        }
	        else
	        {
			$server->command ( "notice $nick Unable to add the quote.\n" );
	                $add_it = ('0');
	        }
	        $sth->finish;
	        }
	        else
	        {
	                $server->command ( "notice $nick Unable to add the quote. Must be longer than 2 chars!\n" );
	                $add_it = ('0');
	        }
	}
	}
}

command_bind(
    delquote => sub {

        my ($deleted) = @_;

        $dbh = DBI->connect("DBI:mysql:$database;mysql_socket=$socket","$username","$password")
        or die "Could not connect to database: " . DBI->errstr;

        $sth = $dbh->prepare('UPDATE irssi_quotes set deleted = "1" WHERE qid = ? AND deleted = 0');

        $rv = $sth->execute($deleted) or die "Can not execute the query: $sth->errstr\n";

        if ($rv >= 1)
        {
                active_win->print("Quote with id $deleted deleted.");
        }
        else
        {
                active_win->print("Unable to delete the quote.");
        }
        $sth->finish;

    }
);

command_bind(
    restore => sub {

        my ($restored) = @_;

        $dbh = DBI->connect("DBI:mysql:$database;mysql_socket=$socket","$username","$password")
        or die "Could not connect to database: " . DBI->errstr;

        $sth = $dbh->prepare('UPDATE irssi_quotes set deleted = "0" WHERE qid = ? AND deleted = 1');

        $rv = $sth->execute($restored) or die "Can not execute the query: $sth->errstr\n";

        if ($rv >= 1)
        {
                active_win->print("Quote with id $restored restored.");
        }
        else
        {
                active_win->print("Unable to restore the quote with id $restored.");
        }
        $sth->finish;

    }
);

Irssi::signal_add('event privmsg', 'event_privmsg');
